@@ -107,7 +107,7 @@ writeheader(FILE *fp)
 	fprintf(fp, "<table>\n<tr><td><img src=\"%slogo.png\" alt=\"\" width=\"32\" height=\"32\" /></td>\n"
 	        "<td><h1>%s</h1><span class=\"desc\">%s</span></td></tr><tr><td></td><td>\n",
 		relpath, name, description);
-	fputs("</td></tr>\n</table>\n<hr/><div id=\"content\">\n"
+	fputs("</td></tr>\n</table>\n<hr/>\n<div id=\"content\">\n"
 	      "<table id=\"index\"><thead>\n"
 	      "<tr><td>Name</td><td>Description</td><td>Owner</td><td>Last commit</td></tr>"
 	      "</thead><tbody>\n", fp);
@@ -118,12 +118,13 @@ writeheader(FILE *fp)
 int
 writefooter(FILE *fp)
 {
-	return !fputs("</tbody></table></div></body>\n</html>", fp);
+	return !fputs("</tbody>\n</table>\n</div>\n</body>\n</html>\n", fp);
 }
 
 int
 writelog(FILE *fp)
 {
+	char *stripped_name = NULL, *p;
 	git_commit *commit = NULL;
 	const git_signature *author;
 	git_revwalk *w = NULL;
@@ -143,10 +144,17 @@ writelog(FILE *fp)
 
 	author = git_commit_author(commit);
 
+	/* strip .git suffix */
+	if (!(stripped_name = strdup(name)))
+		err(1, "strdup");
+	if ((p = strrchr(stripped_name, '.')))
+		if (!strcmp(p, ".git"))
+			*p = '\0';
+
 	fputs("<tr><td><a href=\"", fp);
-	xmlencode(fp, name, strlen(name));
+	xmlencode(fp, stripped_name, strlen(stripped_name));
 	fputs("/log.html\">", fp);
-	xmlencode(fp, name, strlen(name));
+	xmlencode(fp, stripped_name, strlen(stripped_name));
 	fputs("</a></td><td>", fp);
 	xmlencode(fp, description, strlen(description));
 	fputs("</td><td>", fp);
@@ -159,6 +167,7 @@ writelog(FILE *fp)
 	git_commit_free(commit);
 err:
 	git_revwalk_free(w);
+	free(stripped_name);
 
 	return ret;
 }
