@@ -42,7 +42,8 @@ static git_repository *repo;
 static const char *relpath = "";
 static const char *repodir;
 
-static char name[255];
+static char *name;
+static char *stripped_name;
 static char description[255];
 static char cloneurl[1024];
 static int hasreadme, haslicense;
@@ -239,7 +240,7 @@ writeheader(FILE *fp)
 		"<html dir=\"ltr\" lang=\"en\">\n<head>\n"
 		"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\n"
 		"<meta http-equiv=\"Content-Language\" content=\"en\" />\n<title>", fp);
-	xmlencode(fp, name, strlen(name));
+	xmlencode(fp, stripped_name, strlen(stripped_name));
 	if (description[0])
 		fputs(" - ", fp);
 	xmlencode(fp, description, strlen(description));
@@ -251,7 +252,7 @@ writeheader(FILE *fp)
 	fprintf(fp, "<a href=\"../%s\"><img src=\"%slogo.png\" alt=\"\" width=\"32\" height=\"32\" /></a>",
 	        relpath, relpath);
 	fputs("</td><td><h1>", fp);
-	xmlencode(fp, name, strlen(name));
+	xmlencode(fp, stripped_name, strlen(stripped_name));
 	fputs("</h1><span class=\"desc\">", fp);
 	xmlencode(fp, description, strlen(description));
 	fputs("</span></td></tr>", fp);
@@ -270,7 +271,7 @@ writeheader(FILE *fp)
 		fprintf(fp, " | <a href=\"%sfile/README.html\">README</a>", relpath);
 	if (haslicense)
 		fprintf(fp, " | <a href=\"%sfile/LICENSE.html\">LICENSE</a>", relpath);
-	fputs("</td></tr></table>\n<hr/><div id=\"content\">\n", fp);
+	fputs("</td></tr></table>\n<hr/>\n<div id=\"content\">\n", fp);
 
 	return 0;
 }
@@ -278,7 +279,7 @@ writeheader(FILE *fp)
 int
 writefooter(FILE *fp)
 {
-	return !fputs("</div></body>\n</html>", fp);
+	return !fputs("</div>\n</body>\n</html>\n", fp);
 }
 
 void
@@ -317,17 +318,6 @@ printcommit(FILE *fp, struct commitinfo *ci)
 		fprintf(fp, "<b>parent</b> <a href=\"%scommit/%s.html\">%s</a>\n",
 			relpath, ci->parentoid, ci->parentoid);
 
-#if 0
-	if ((count = (int)git_commit_parentcount(commit)) > 1) {
-		fputs("<b>Merge:</b>", fp);
-		for (i = 0; i < count; i++) {
-			git_oid_tostr(buf, 8, git_commit_parent_id(commit, i));
-			fprintf(fp, " <a href=\"%scommit/%s.html\">%s</a>",
-				relpath, buf, buf);
-		}
-		fputc('\n', fp);
-	}
-#endif
 	if (ci->author) {
 		fputs("<b>Author:</b> ", fp);
 		xmlencode(fp, ci->author->name, strlen(ci->author->name));
@@ -531,18 +521,6 @@ printcommitatom(FILE *fp, struct commitinfo *ci)
 	fprintf(fp, "commit %s\n", ci->oid);
 	if (ci->parentoid[0])
 		fprintf(fp, "parent %s\n", ci->parentoid);
-
-#if 0
-	if ((count = (int)git_commit_parentcount(commit)) > 1) {
-		fputs("Merge:", fp);
-		for (i = 0; i < count; i++) {
-			git_oid_tostr(buf, 8, git_commit_parent_id(commit, i));
-			fprintf(fp, " %s", buf);
-		}
-		fputc('\n', fp);
-	}
-#endif
-
 	if (ci->author) {
 		fputs("Author: ", fp);
 		xmlencode(fp, ci->author->name, strlen(ci->author->name));
@@ -550,11 +528,12 @@ printcommitatom(FILE *fp, struct commitinfo *ci)
 		xmlencode(fp, ci->author->email, strlen(ci->author->email));
 		fputs("&gt;\nDate:   ", fp);
 		printtime(fp, &(ci->author->when));
+		fputc('\n', fp);
 	}
-	fputc('\n', fp);
-
-	if (ci->msg)
+	if (ci->msg) {
+		fputc('\n', fp);
 		xmlencode(fp, ci->msg, strlen(ci->msg));
+	}
 	fputs("\n</content>\n", fp);
 	if (ci->author) {
 		fputs("<author><name>", fp);
@@ -576,9 +555,8 @@ writeatom(FILE *fp)
 
 	fputs("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
 	      "<feed xmlns=\"http://www.w3.org/2005/Atom\">\n<title>", fp);
-	xmlencode(fp, name, strlen(name));
-	fputs(", branch master</title>\n<subtitle>", fp);
-
+	xmlencode(fp, stripped_name, strlen(stripped_name));
+	fputs(", branch HEAD</title>\n<subtitle>", fp);
 	xmlencode(fp, description, strlen(description));
 	fputs("</subtitle>\n", fp);
 
@@ -628,7 +606,7 @@ writeblob(git_object *obj, const char *filename, git_off_t filesize)
 	writeheader(fp);
 	fputs("<p> ", fp);
 	xmlencode(fp, filename, strlen(filename));
-	fprintf(fp, " (%" PRIu32 "b)", filesize);
+	fprintf(fp, " (%jub)", (uintmax_t)filesize);
 	fputs("</p><hr/>", fp);
 
 	if (git_blob_is_binary((git_blob *)obj)) {
@@ -734,7 +712,7 @@ writefilestree(FILE *fp, git_tree *tree, const char *branch, const char *path)
 		fputs(".html\">", fp);
 		xmlencode(fp, filename, strlen(filename));
 		fputs("</a></td><td class=\"num\">", fp);
-		fprintf(fp, "%" PRIu32, filesize);
+		fprintf(fp, "%ju", (uintmax_t)filesize);
 		fputs("</td></tr>\n", fp);
 
 		writeblob(obj, filename, filesize);
@@ -775,183 +753,141 @@ err:
 }
 
 int
-writebranches(FILE *fp)
+refs_cmp(const void *v1, const void *v2)
 {
-	struct commitinfo *ci;
-	git_branch_iterator *it = NULL;
-	git_branch_t branch;
-	git_reference *ref = NULL, *dref = NULL;
-	const git_oid *id = NULL;
-	const char *branchname = NULL;
-	size_t len;
-	int ret = -1;
+	git_reference *r1 = (*(git_reference **)v1);
+	git_reference *r2 = (*(git_reference **)v2);
+	int t1, t2;
 
-	/* log for local branches */
-	if (git_branch_iterator_new(&it, repo, GIT_BRANCH_LOCAL))
-		return -1;
+	t1 = git_reference_is_branch(r1);
+	t2 = git_reference_is_branch(r2);
 
-	fputs("<h2>Branches</h2><table id=\"branches\"><thead>\n<tr><td>Branch</td><td>Age</td>"
-		  "<td>Commit message</td>"
-		  "<td>Author</td><td>Files</td><td class=\"num\">+</td>"
-		  "<td class=\"num\">-</td></tr>\n</thead><tbody>\n", fp);
+	if (t1 != t2)
+		return t1 - t2;
 
-	while (!git_branch_next(&ref, &branch, it)) {
-		if (git_branch_name(&branchname, ref))
-			continue;
-
-		id = NULL;
-		switch (git_reference_type(ref)) {
-		case GIT_REF_SYMBOLIC:
-			if (git_reference_resolve(&dref, ref))
-				goto err;
-			id = git_reference_target(dref);
-			break;
-		case GIT_REF_OID:
-			id = git_reference_target(ref);
-			break;
-		default:
-			continue;
-		}
-		if (!id)
-			goto err;
-		if (!(ci = commitinfo_getbyoid(id)))
-			break;
-
-		relpath = "";
-
-		fputs("<tr><td>", fp);
-		xmlencode(fp, branchname, strlen(branchname));
-		fputs("</td><td>", fp);
-		if (ci->author)
-			printtimeshort(fp, &(ci->author->when));
-		fputs("</td><td>", fp);
-		if (ci->summary) {
-			if ((len = strlen(ci->summary)) > summarylen) {
-				xmlencode(fp, ci->summary, summarylen - 1);
-				fputs("…", fp);
-			} else {
-				xmlencode(fp, ci->summary, len);
-			}
-		}
-		fputs("</td><td>", fp);
-		if (ci->author)
-			xmlencode(fp, ci->author->name, strlen(ci->author->name));
-		fputs("</td><td class=\"num\">", fp);
-		fprintf(fp, "%zu", ci->filecount);
-		fputs("</td><td class=\"num\">", fp);
-		fprintf(fp, "+%zu", ci->addcount);
-		fputs("</td><td class=\"num\">", fp);
-		fprintf(fp, "-%zu", ci->delcount);
-		fputs("</td></tr>\n", fp);
-
-		relpath = "../";
-
-		commitinfo_free(ci);
-		git_reference_free(ref);
-		git_reference_free(dref);
-		ref = NULL;
-		dref = NULL;
-	}
-	ret = 0;
-
-err:
-	fputs("</tbody></table>", fp);
-	git_reference_free(ref);
-	git_reference_free(dref);
-	git_branch_iterator_free(it);
-
-	return ret;
-}
-
-int
-tagcompare(void *s1, void *s2)
-{
-	return strcmp(*(char **)s1, *(char **)s2);
-}
-
-int
-writetags(FILE *fp)
-{
-	struct commitinfo *ci;
-	git_strarray tagnames;
-	git_object *obj = NULL;
-	const git_oid *id = NULL;
-	size_t i, len;
-
-	/* summary page with branches and tags */
-	memset(&tagnames, 0, sizeof(tagnames));
-	if (git_tag_list(&tagnames, repo))
-		return -1;
-	if (!tagnames.count) {
-		git_strarray_free(&tagnames);
-		return 0;
-	}
-
-	/* sort names */
-	qsort(tagnames.strings, tagnames.count, sizeof(char *),
-	      (int (*)(const void *, const void *))&tagcompare);
-
-	fputs("<h2>Tags</h2><table id=\"tags\"><thead>\n<tr><td>Tag</td>"
-	      "<td>Age</td><td>Commit message</td>"
-	      "<td>Author</td><td>Files</td><td class=\"num\">+</td>"
-	      "<td class=\"num\">-</td></tr>\n</thead><tbody>\n", fp);
-
-	for (i = 0; i < tagnames.count; i++) {
-		if (git_revparse_single(&obj, repo, tagnames.strings[i]))
-			continue;
-		id = git_object_id(obj);
-		if (!(ci = commitinfo_getbyoid(id)))
-			break;
-
-		relpath = "";
-
-		fputs("<tr><td>", fp);
-		xmlencode(fp, tagnames.strings[i], strlen(tagnames.strings[i]));
-		fputs("</td><td>", fp);
-		if (ci->author)
-			printtimeshort(fp, &(ci->author->when));
-		fputs("</td><td>", fp);
-		if (ci->summary) {
-			fprintf(fp, "<a href=\"%scommit/%s.html\">", relpath, ci->oid);
-			if ((len = strlen(ci->summary)) > summarylen) {
-				xmlencode(fp, ci->summary, summarylen - 1);
-				fputs("…", fp);
-			} else {
-				xmlencode(fp, ci->summary, len);
-			}
-			fputs("</a>", fp);
-		}
-		fputs("</td><td>", fp);
-		if (ci->author)
-			xmlencode(fp, ci->author->name, strlen(ci->author->name));
-		fputs("</td><td class=\"num\">", fp);
-		fprintf(fp, "%zu", ci->filecount);
-		fputs("</td><td class=\"num\">", fp);
-		fprintf(fp, "+%zu", ci->addcount);
-		fputs("</td><td class=\"num\">", fp);
-		fprintf(fp, "-%zu", ci->delcount);
-		fputs("</td></tr>\n", fp);
-
-		relpath = "../";
-
-		commitinfo_free(ci);
-		git_object_free(obj);
-	}
-	fputs("</tbody></table>", fp);
-	git_strarray_free(&tagnames);
-
-	return 0;
+	return strcmp(git_reference_shorthand(r1),
+	              git_reference_shorthand(r2));
 }
 
 int
 writerefs(FILE *fp)
 {
-	int ret;
+	struct commitinfo *ci;
+	const git_oid *id = NULL;
+	git_object *obj = NULL;
+	git_reference *dref = NULL, *r, *ref = NULL;
+	git_reference_iterator *it = NULL;
+	git_reference **refs = NULL;
+	size_t count, i, j, len, refcount = 0;
+	const char *cols[] = { "Branch", "Tag" }; /* first column title */
+	const char *titles[] = { "Branches", "Tags" };
+	const char *ids[] = { "branches", "tags" };
+	const char *name;
 
-	if ((ret = writebranches(fp)))
-		return ret;
-	fputs("<br/>", fp);
-	return writetags(fp);
+	if (git_reference_iterator_new(&it, repo))
+		return -1;
+
+	for (refcount = 0; !git_reference_next(&ref, it); refcount++) {
+		if (!(refs = reallocarray(refs, refcount + 1, sizeof(git_reference *))))
+			err(1, "realloc");
+		refs[refcount] = ref;
+	}
+	git_reference_iterator_free(it);
+
+	/* sort by type then shorthand name */
+	qsort(refs, refcount, sizeof(git_reference *), refs_cmp);
+
+	for (j = 0; j < 2; j++) {
+		for (i = 0, count = 0; i < refcount; i++) {
+			if (!(git_reference_is_branch(refs[i]) && j == 0) &&
+			    !(git_reference_is_tag(refs[i]) && j == 1))
+				continue;
+
+			switch (git_reference_type(refs[i])) {
+			case GIT_REF_SYMBOLIC:
+				if (git_reference_resolve(&dref, refs[i]))
+					goto err;
+				r = dref;
+				break;
+			case GIT_REF_OID:
+				r = refs[i];
+				break;
+			default:
+				continue;
+			}
+			if (!(id = git_reference_target(r)))
+				goto err;
+			if (git_reference_peel(&obj, r, GIT_OBJ_ANY))
+				goto err;
+			if (!(id = git_object_id(obj)))
+				goto err;
+			if (!(ci = commitinfo_getbyoid(id)))
+				break;
+
+			/* print header if it has an entry (first). */
+			if (++count == 1) {
+				fprintf(fp, "<h2>%s</h2><table id=\"%s\"><thead>\n<tr><td>%s</td>"
+				      "<td>Age</td><td>Commit message</td>"
+				      "<td>Author</td><td>Files</td><td class=\"num\">+</td>"
+				      "<td class=\"num\">-</td></tr>\n</thead><tbody>\n",
+				      titles[j], ids[j], cols[j]);
+			}
+
+			relpath = "";
+			name = git_reference_shorthand(r);
+
+			fputs("<tr><td>", fp);
+			xmlencode(fp, name, strlen(name));
+			fputs("</td><td>", fp);
+			if (ci->author)
+				printtimeshort(fp, &(ci->author->when));
+			fputs("</td><td>", fp);
+			if (ci->summary) {
+				if (j)
+					fprintf(fp, "<a href=\"%scommit/%s.html\">",
+					        relpath, ci->oid);
+				if ((len = strlen(ci->summary)) > summarylen) {
+					xmlencode(fp, ci->summary, summarylen - 1);
+					fputs("…", fp);
+				} else {
+					xmlencode(fp, ci->summary, len);
+				}
+				if (j)
+					fputs("</a>", fp);
+			}
+			fputs("</td><td>", fp);
+			if (ci->author)
+				xmlencode(fp, ci->author->name, strlen(ci->author->name));
+			fputs("</td><td class=\"num\">", fp);
+			fprintf(fp, "%zu", ci->filecount);
+			fputs("</td><td class=\"num\">", fp);
+			fprintf(fp, "+%zu", ci->addcount);
+			fputs("</td><td class=\"num\">", fp);
+			fprintf(fp, "-%zu", ci->delcount);
+			fputs("</td></tr>\n", fp);
+
+			relpath = "../";
+
+			commitinfo_free(ci);
+			git_object_free(obj);
+			obj = NULL;
+			git_reference_free(dref);
+			dref = NULL;
+		}
+		/* table footer */
+		if (count)
+			fputs("</tbody></table>", fp);
+	}
+
+err:
+	git_object_free(obj);
+	git_reference_free(dref);
+
+	for (i = 0; i < refcount; i++)
+		git_reference_free(refs[i]);
+	free(refs);
+
+	return 0;
 }
 
 int
@@ -979,9 +915,14 @@ main(int argc, char *argv[])
 	}
 
 	/* use directory name as name */
-	p = xbasename(repodir);
-	snprintf(name, sizeof(name), "%s", p);
-	free(p);
+	name = xbasename(repodir);
+
+	/* strip .git suffix */
+	if (!(stripped_name = strdup(name)))
+		err(1, "strdup");
+	if ((p = strrchr(stripped_name, '.')))
+		if (!strcmp(p, ".git"))
+			*p = '\0';
 
 	/* read description or .git/description */
 	snprintf(path, sizeof(path), "%s%s%s",
@@ -1020,7 +961,6 @@ main(int argc, char *argv[])
 	git_object_free(obj);
 
 	/* log for HEAD */
-	mkdir("log", 0755);
 	fp = efopen("log.html", "w");
 	relpath = "";
 	writeheader(fp);
